/*
 * UbuntuOne Nautilus plugin
 *
 * Authors: Rodrigo Moya <rodrigo.moya@canonical.com>
 *
 * Copyright 2009-2010 Canonical Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3, as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <glib/gi18n.h>
#include <libsyncdaemon/syncdaemon-folders-interface.h>
#include <libsyncdaemon/syncdaemon-shares-interface.h>
#include "location-widget.h"

G_DEFINE_TYPE(LocationWidget, location_widget, GTK_TYPE_HBOX)

static void
set_label_text (LocationWidget *location, gboolean enabled)
{
	gchar *labeltext;

	labeltext = g_strdup_printf ("<b>Ubuntu One</b> %s",
				     enabled ? _("enabled") : _("disabled"));
	gtk_label_set_markup (GTK_LABEL (location->info_label), labeltext);
	g_free (labeltext);
}

static void
folder_created_cb (SyncdaemonDaemon *daemon, gboolean success, SyncdaemonFolderInfo *folder_info, gpointer user_data)
{
	const gchar *path;
	LocationWidget *location = LOCATION_WIDGET (user_data);

	path = syncdaemon_folder_info_get_path (folder_info);
	g_warning ("in folder_created_cb with path = %s", path);
	if (g_strcmp0 (path, location->path) == 0) {
		set_label_text (location, success);
		if (success) {
			gtk_widget_hide (location->enable_button);
			gtk_widget_show (location->disable_button);
			gtk_widget_set_sensitive (location->disable_button, TRUE);
		} else {
			gtk_widget_show (location->enable_button);
			gtk_widget_set_sensitive (location->enable_button, TRUE);
			gtk_widget_hide (location->disable_button);
			ubuntuone_show_error_dialog (location->uon, _("Error enabling folder"),
						     _("Could not enable folder %s for synchronizing to Ubuntu One"),
						     location->path);
		}
	}
}

static void
folder_deleted_cb (SyncdaemonDaemon *daemon, gboolean success, SyncdaemonFolderInfo *folder_info, gpointer user_data)
{
	const gchar *path;
	LocationWidget *location = LOCATION_WIDGET (user_data);

	path = syncdaemon_folder_info_get_path (folder_info);
	if (g_strcmp0 (path, location->path) == 0) {
		if (success) {
			set_label_text (location, FALSE);
			gtk_widget_show (location->enable_button);
			gtk_widget_set_sensitive (location->enable_button, TRUE);
			gtk_widget_hide (location->disable_button);
		} else {
			set_label_text (location, TRUE);
			gtk_widget_hide (location->enable_button);
			gtk_widget_show (location->disable_button);
			gtk_widget_set_sensitive (location->disable_button, TRUE);
			ubuntuone_show_error_dialog (location->uon, _("Error disabling folder"),
						     _("Could not disable folder %s for synchronizing to Ubuntu One"),
						     location->path);
		}
	}
}

static void
location_widget_finalize (GObject *object)
{
	LocationWidget *location = LOCATION_WIDGET (object);

	g_signal_handlers_disconnect_by_func (location->uon->syncdaemon, folder_created_cb, location);
	g_signal_handlers_disconnect_by_func (location->uon->syncdaemon, folder_deleted_cb, location);

	if (location->path != NULL)
		g_free (location->path);

	G_OBJECT_CLASS (location_widget_parent_class)->finalize (object);
}

static void
location_widget_class_init (LocationWidgetClass *klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS (klass);

	object_class->finalize = location_widget_finalize;
}

static void
disable_u1_cb (GtkButton *button, gpointer user_data)
{
	SyncdaemonInterface *interface;
	gchar *labeltext;
	LocationWidget *location = LOCATION_WIDGET (user_data);

	/* Update label to show progress */
	labeltext = g_strdup_printf (_("Disabling %s folder for synchronization on Ubuntu One..."), location->path);
	gtk_label_set_text (GTK_LABEL (location->info_label), labeltext);
	g_free (labeltext);
	gtk_widget_set_sensitive (location->disable_button, FALSE);

	/* Perform the removal of this folder */
	interface = syncdaemon_daemon_get_folders_interface (location->uon->syncdaemon);
	if (interface != NULL) {
		SyncdaemonFolderInfo *folder_info;

		folder_info = syncdaemon_folders_interface_get_info (
			SYNCDAEMON_FOLDERS_INTERFACE (interface),
			location->path);
		if (folder_info != NULL) {
			syncdaemon_folders_interface_delete (
				SYNCDAEMON_FOLDERS_INTERFACE (interface),
				syncdaemon_folder_info_get_volume_id (folder_info));

			g_object_unref (G_OBJECT (folder_info));
		}
	}
}

static void
enable_u1_cb (GtkButton *button, gpointer user_data)
{
	SyncdaemonInterface *interface;
	gchar *labeltext;
	LocationWidget *location = LOCATION_WIDGET (user_data);

	/* Update label to show progress */
	labeltext = g_strdup_printf (_("Enabling %s folder for synchronization on Ubuntu One..."), location->path);
	gtk_label_set_text (GTK_LABEL (location->info_label), labeltext);
	g_free (labeltext);
	gtk_widget_set_sensitive (location->enable_button, FALSE);

	/* Perform the addition of this folder */
	interface = syncdaemon_daemon_get_folders_interface (location->uon->syncdaemon);
	if (interface != NULL) {
		syncdaemon_folders_interface_create (SYNCDAEMON_FOLDERS_INTERFACE (interface),
						     location->path);
	}
}

static void
location_widget_init (LocationWidget *location)
{
	location->path = NULL;
	location->info_label = location->enable_button = NULL;

	/* Create the widgets in the interface */
	location->info_label = gtk_label_new ("<b>Ubuntu One</b>");
	gtk_label_set_use_markup (GTK_LABEL (location->info_label), TRUE);
	gtk_misc_set_alignment (GTK_MISC (location->info_label), 0.0, 0.5);
	gtk_widget_show (location->info_label);
	gtk_box_pack_start (GTK_BOX (location), location->info_label, TRUE, FALSE, 0);

	location->disable_button = gtk_button_new_with_label (_("Disable"));
	g_signal_connect (G_OBJECT (location->disable_button), "clicked",
			  G_CALLBACK (disable_u1_cb), location);
	gtk_box_pack_end (GTK_BOX (location), location->disable_button, FALSE, FALSE, 0);

	location->enable_button = gtk_button_new_with_label (_("Enable"));
	g_signal_connect (G_OBJECT (location->enable_button), "clicked",
			  G_CALLBACK (enable_u1_cb), location);
	gtk_box_pack_end (GTK_BOX (location), location->enable_button, FALSE, FALSE, 0);
}

GtkWidget *
location_widget_new (UbuntuOneNautilus *uon, const gchar *path)
{
	LocationWidget *location;
	gboolean is_root;

	g_return_val_if_fail (uon != NULL, NULL);
	g_return_val_if_fail (path != NULL, NULL);

	location = (LocationWidget *) g_object_new (TYPE_LOCATION_WIDGET, NULL);
	location->uon = uon;
	location->path = g_strdup (path);

	/* Connect to Syncdaemon signals */
	g_signal_connect (G_OBJECT (uon->syncdaemon), "folder_created",
			  G_CALLBACK (folder_created_cb), location);
	g_signal_connect (G_OBJECT (uon->syncdaemon), "folder_deleted",
			  G_CALLBACK (folder_deleted_cb), location);

	if (ubuntuone_is_storagefs (uon, path, &is_root)) {
		/* Create label and disable button */
		set_label_text (location, TRUE);
		if (is_root && strcmp (path, uon->managed) != 0)
			gtk_widget_show (location->disable_button);
	} else {
		set_label_text (location, FALSE);
		gtk_widget_show (location->enable_button);
	}

	gtk_widget_show (GTK_WIDGET (location));

	return GTK_WIDGET (location);
}
